@@ -26,7 +26,7 @@
 
 
 using System;
-using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Text;
 namespace Beagle.Filters {
@@ -46,7 +46,6 @@ namespace Beagle.Filters {
 		};
 		
 		protected LangType SrcLangType;
-		protected Hashtable KeyWordsHash;
 
 		private enum LineType {
 			None,
@@ -57,8 +56,8 @@ namespace Beagle.Filters {
 		
 		LineType SrcLineType;
 		string StrConstIdentifier;
-
 		StringBuilder token;
+
 		public FilterSource ()
 		{
 			// Initialize the linetype member.
@@ -66,13 +65,30 @@ namespace Beagle.Filters {
 			SrcLangType = LangType.None;
 			StrConstIdentifier = " ";
 
-			KeyWordsHash = new Hashtable ();
-
 			SnippetMode = true;
 			OriginalIsText = true;
 			token = new StringBuilder ();
 		}
 
+		protected abstract Dictionary<string, bool> KeyWordsHash {
+			get;
+		}
+
+		override protected void DoPull ()
+		{
+			string str = TextReader.ReadLine ();
+			if (str == null)
+				Finished ();
+			else
+				ExtractTokens (str);
+		}
+
+		protected override void DoClose ()
+		{
+			token.Length = 0;
+			token = null;
+		}
+
 		// Validate the character and append it to the token,
 		// that will be added to the text-pool of the filter.
 		// Returns: False, if token is not complete enough to be added to the pool.
@@ -82,18 +98,15 @@ namespace Beagle.Filters {
 			if (ch == ' ')
 				return true;
 
-			if (Char.IsLetter (ch) ||
-			    Char.IsDigit (ch) ||
-			    ch == '_') {
+			if (Char.IsLetter (ch) || Char.IsDigit (ch) || ch == '_') {
 				token.Append (ch);
 				if ((index + 1) < length) 
 					return false;
 				else
 					return true;
-			} else
-				return true;
+			}
 
-			return false;
+			return true;
 		}
 
 		// Tokenize the passed string and add the relevant 
@@ -105,7 +118,7 @@ namespace Beagle.Filters {
 		{
 			int index;
 			token.Length = 0;
-			string splCharSeq = "";
+			string splCharSeq = String.Empty;
 
 			for (index = 0; index < str.Length; index++) {
 		         	if (((str[index] == '{' 
@@ -130,7 +143,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						    } else 
 							token.Append (splCharSeq);
-						splCharSeq = ""; 
+						splCharSeq = String.Empty; 
                                                 break;
 
 					case "*)":
@@ -141,7 +154,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
-						splCharSeq = "";
+						splCharSeq = String.Empty;
 						break;          
                            
                                         case "{":
@@ -150,7 +163,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						    } else 
 							token.Append (splCharSeq);
-						splCharSeq = ""; 
+						splCharSeq = String.Empty; 
                                                 break;
 
 					case "}":
@@ -161,7 +174,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
-						splCharSeq = "";
+						splCharSeq = String.Empty;
 						break;
 					case "//":
 						if (SrcLineType == LineType.None) {
@@ -169,7 +182,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						} else
 							token.Append (splCharSeq);
-						splCharSeq = "";
+						splCharSeq = String.Empty;
 						break;
 						
 					case "/*":
@@ -178,7 +191,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						} else 
 							token.Append (splCharSeq);
-						splCharSeq = "";
+						splCharSeq = String.Empty;
 						break;
 							
 					case "*/":
@@ -189,7 +202,7 @@ namespace Beagle.Filters {
 							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
-						splCharSeq = "";
+						splCharSeq = String.Empty;
 						break;
 					}
 				} else if ((str[index] == '#' && (SrcLangType == LangType.Python_Style ||
@@ -231,7 +244,7 @@ namespace Beagle.Filters {
 						index += 2;
 					}
 					       
-					splCharSeq = "";
+					splCharSeq = String.Empty;
 				}
 				// Lisp: ignore the single quote character; do another iteration
 				else if (SrcLangType == LangType.Lisp_Style && str[index] == '\'') {
@@ -254,12 +267,12 @@ namespace Beagle.Filters {
 						token.Length = 0;
 					} else
 						token.Append (str[index]);
-					splCharSeq = "";
+					splCharSeq = String.Empty;
 
 				} else if (SrcLineType != LineType.None) {
 					token.Append (splCharSeq);
 					token.Append (str[index]);
-					splCharSeq = "";
+					splCharSeq = String.Empty;
 
 				} else if (SrcLineType == LineType.None) { 
 					if (AppendToToken (str[index], index, str.Length)) {			       
@@ -277,14 +290,14 @@ namespace Beagle.Filters {
 								continue;
 							}
 						}
-						//token = token.Replace(" ", "");
+						//token = token.Replace(" ", String.Empty);
 						if (token.Length > 0) {
 							string tok;
 							if (SrcLangType == LangType.Fortran_Style)
 								tok = token.ToString().ToLower();
 							else
 								tok = token.ToString ();
-							if (!KeyWordsHash.Contains (tok)) {
+							if (! KeyWordsHash.ContainsKey (tok)) {
 								if (!Char.IsDigit (token[0])) {
 									AppendText (tok);
 									AppendWhiteSpace ();
@@ -294,7 +307,7 @@ namespace Beagle.Filters {
 						// reset the token
 						token.Length = 0;
 					}
-					splCharSeq = "";
+					splCharSeq = String.Empty;
 				}
 		        }
 			if (SrcLineType != LineType.None) {
